use std::collections::HashMap;

#[derive(Debug, serde::Deserialize)]
#[serde(deny_unknown_fields)]
struct AccountSpec {
    title: Option<String>,
    description: Option<String>,

    annotations: Option<HashMap<String, String>>,
}

#[derive(Debug, serde::Deserialize)]
#[serde(deny_unknown_fields)]
struct RootSpec {
    accounts: HashMap<String, AccountSpec>,

    account_data: HashMap<String, AccountData>,
}

pub struct Root {
    path: std::path::PathBuf,
    data: RootSpec,
}

impl Root {
    pub fn load(path: &std::path::Path) -> std::io::Result<Self> {
        let content = std::fs::read_to_string(path)?;
        let data = toml::from_str::<RootSpec>(&content).expect("could not parse root");
        Ok(Self {
            path: path.into(),
            data,
        })
    }

    pub fn iter_accounts(&self) -> impl Iterator<Item = (&str, &AccountSpec)> {
        self.data.accounts.iter().map(|v| (v.0.as_str(), v.1))
    }
}
